import { Observable, Subject } from 'rxjs';

import {
  RxWebsocketSubject,
  RxWebsocketSubjectConfig,
} from './RxWebsocketSubject';

import { RPCClientRWC } from '../../RPCClientRWC';
import { RPCMessage } from '../../../core/type';


export class RPCClientWebsocketRWC implements RPCClientRWC {
  private _wsSocketSubject: RxWebsocketSubject;
  private _responseSubject: Subject<RPCMessage>;

  public constructor(
    private _config: RxWebsocketSubjectConfig,
  ) {
    this._wsSocketSubject = new RxWebsocketSubject(this._config);
  }

  public connect(queryString?: string): void {
    if (undefined !== queryString) {
      this._wsSocketSubject.queryString = queryString;
    }
    this._wsSocketSubject.connect();
    this._wsSocketSubject.subscribe(
      (value: RPCMessage) => {
        if (undefined !== this._responseSubject) {
          this._responseSubject.next(value);
        }
      },
      (error: any) => {
        if (undefined !== this._responseSubject) {
          this._responseSubject.error(error);
        }
      },
      () => {
        console.log('sss');
      }
    );
  }

  public disconnect(): void {
    this._wsSocketSubject.disconnect();
  }

  public connectionStatus(): Observable<boolean> {
    return this._wsSocketSubject.connectionStatus;
  }

  public readResponse(): Observable<RPCMessage> {
    if (undefined === this._responseSubject) {
      this._responseSubject = new Subject<RPCMessage>();
    }
    return this._responseSubject.asObservable();
  }

  public writeRequest(data: RPCMessage): void {
    this._wsSocketSubject.write(data);
  }
}
